@@ -279,9 +279,10 @@ unix_open_file (const char  *filename,
 
 typedef struct
 {
-  gint             fds[3];
-  GSpawnChildSetupFunc child_setup_func;
-  gpointer             child_setup_data;
+  gint                   fds[3];
+  GArray                *inherit_fds;
+  GSpawnChildSetupFunc   child_setup_func;
+  gpointer               child_setup_data;
 } ChildData;
 
 static void
@@ -289,6 +290,7 @@ child_setup (gpointer user_data)
 {
   ChildData *child_data = user_data;
   gint i;
+  gint result;
 
   /* We're on the child side now.  "Rename" the file descriptors in
    * child_data.fds[] to stdin/stdout/stderr.
@@ -298,14 +300,31 @@ child_setup (gpointer user_data)
    * have been created O_CLOEXEC.
    */
   for (i = 0; i < 3; i++)
-    if (child_data->fds[i] != -1 && child_data->fds[i] != i)
-      {
-        gint result;
+    {
+      if (child_data->fds[i] != -1 && child_data->fds[i] != i)
+        {
+          do
+            result = dup2 (child_data->fds[i], i);
+          while (G_UNLIKELY (result == -1 && errno == EINTR));
+        }
+    }
 
-        do
-          result = dup2 (child_data->fds[i], i);
-        while (result == -1 && errno == EINTR);
-      }
+  /* Unset the CLOEXEC flag for the child *should* inherit */
+  for (i = 0; i < child_data->inherit_fds->len; i++)
+    {
+      int fd = g_array_index (child_data->inherit_fds, int, i);
+      int flags;
+
+      do
+        flags = fcntl (fd, F_GETFL);
+      while (G_UNLIKELY (flags == -1 && errno == EINTR));
+
+      flags &= ~FD_CLOEXEC;
+      
+      do
+        result = fcntl (fd, F_SETFD, flags);
+      while (G_UNLIKELY (result == -1 && errno == EINTR));
+    }
 
   if (child_data->child_setup_func)
     child_data->child_setup_func (child_data->child_setup_data);
@@ -401,6 +420,8 @@ initable_init (GInitable     *initable,
   else
     g_assert_not_reached ();
 
+  child_data.inherit_fds = self->context->inherit_fds;
+
   if (self->context->keep_descriptors)
     spawn_flags |= G_SPAWN_LEAVE_DESCRIPTORS_OPEN;
 
@@ -437,6 +458,9 @@ out:
     if (close_fds[i] != -1)
       close (close_fds[i]);
 
+  for (i = 0; i < self->context->postfork_close_fds->len; i++)
+    (void) close (g_array_index (self->context->postfork_close_fds, int, i));
+
   self->stdin_pipe = platform_output_stream_from_spawn_fd (pipe_fds[0]);
   self->stdout_pipe = platform_input_stream_from_spawn_fd (pipe_fds[1]);
   self->stderr_pipe = platform_input_stream_from_spawn_fd (pipe_fds[2]);
