@@ -45,7 +45,7 @@ GFile *       ostree_repo_get_path (OstreeRepo  *self);
 typedef enum {
   OSTREE_REPO_MODE_BARE,
   OSTREE_REPO_MODE_ARCHIVE,
-  OSTREE_REPO_MODE_ARCHIVE_Z
+  OSTREE_REPO_MODE_ARCHIVE_Z2
 } OstreeRepoMode;
 
 gboolean       ostree_repo_mode_from_string (const char      *mode,
@@ -77,6 +77,7 @@ GFile *       ostree_repo_get_file_object_path (OstreeRepo   *self,
                                                 const char   *object);
 
 gboolean      ostree_repo_prepare_transaction (OstreeRepo     *self,
+                                               gboolean        enable_commit_hardlink_scan,
                                                GCancellable   *cancellable,
                                                GError        **error);
 
@@ -103,6 +104,19 @@ gboolean      ostree_repo_stage_metadata (OstreeRepo        *self,
                                           GCancellable      *cancellable,
                                           GError           **error);
 
+void          ostree_repo_stage_metadata_async (OstreeRepo              *self,
+                                                OstreeObjectType         objtype,
+                                                const char              *expected_checksum,
+                                                GVariant                *object,
+                                                GCancellable            *cancellable,
+                                                GAsyncReadyCallback      callback,
+                                                gpointer                 user_data);
+
+gboolean      ostree_repo_stage_metadata_finish (OstreeRepo        *self,
+                                                 GAsyncResult      *result,
+                                                 guchar           **out_csum,
+                                                 GError           **error);
+
 gboolean      ostree_repo_stage_content (OstreeRepo       *self,
                                          const char       *expected_checksum,
                                          GInputStream     *content,
@@ -125,6 +139,19 @@ gboolean      ostree_repo_stage_content_trusted (OstreeRepo       *self,
                                                  GCancellable     *cancellable,
                                                  GError          **error);
 
+void          ostree_repo_stage_content_async (OstreeRepo              *self,
+                                               const char              *expected_checksum,
+                                               GInputStream            *object,
+                                               guint64                  file_object_length,
+                                               GCancellable            *cancellable,
+                                               GAsyncReadyCallback      callback,
+                                               gpointer                 user_data);
+
+gboolean      ostree_repo_stage_content_finish (OstreeRepo        *self,
+                                                GAsyncResult      *result,
+                                                guchar           **out_csum,
+                                                GError           **error);
+
 gboolean      ostree_repo_resolve_rev (OstreeRepo  *self,
                                        const char  *rev,
                                        gboolean     allow_noent,
@@ -254,6 +281,10 @@ ostree_repo_checkout_tree_finish (OstreeRepo               *self,
                                   GAsyncResult             *result,
                                   GError                  **error);
 
+gboolean       ostree_repo_checkout_gc (OstreeRepo        *self,
+                                        GCancellable      *cancellable,
+                                        GError           **error);
+
 gboolean       ostree_repo_read_commit (OstreeRepo *self,
                                         const char *rev,
                                         GFile       **out_root,
