@@ -29,10 +29,12 @@
 #include "ot-main.h"
 #include "otutil.h"
 
-static int
-usage (char **argv, OstreeBuiltin *builtins, gboolean is_error)
+int
+ostree_usage (char **argv,
+              OstreeCommand *commands,
+              gboolean is_error)
 {
-  OstreeBuiltin *builtin = builtins;
+  OstreeCommand *command = commands;
   void (*print_func) (const gchar *format, ...);
 
   if (is_error)
@@ -44,20 +46,20 @@ usage (char **argv, OstreeBuiltin *builtins, gboolean is_error)
               argv[0]);
   print_func ("Builtin commands:\n");
 
-  while (builtin->name)
+  while (command->name)
     {
-      print_func ("  %s\n", builtin->name);
-      builtin++;
+      print_func ("  %s\n", command->name);
+      command++;
     }
   return (is_error ? 1 : 0);
 }
 
-static void
-prep_builtin_argv (const char *builtin,
-                   int argc,
-                   char **argv,
-                   int *out_argc,
-                   char ***out_argv)
+void
+ostree_prep_builtin_argv (const char  *builtin,
+                          int          argc,
+                          char       **argv,
+                          int         *out_argc,
+                          char      ***out_argv)
 {
   int i;
   char **cmd_argv;
@@ -72,28 +74,23 @@ prep_builtin_argv (const char *builtin,
   *out_argv = cmd_argv;
 }
 
-static void
-set_error_print_usage (GError **error, OstreeBuiltin *builtins, const char *msg, char **argv)
-{
-  g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, msg);
-  usage (argv, builtins, TRUE);
-}
-
 int
-ostree_main (int    argc,
-             char **argv,
-             OstreeBuiltin  *builtins)
+ostree_run (int    argc,
+            char **argv,
+            OstreeCommand *commands,
+            GError **res_error)
 {
-  OstreeBuiltin *builtin;
+  OstreeCommand *command;
   GError *error = NULL;
   int cmd_argc;
   char **cmd_argv = NULL;
-  gboolean am_root;
   gboolean have_repo_arg;
   const char *binname = NULL;
   const char *slash = NULL;
   const char *cmd = NULL;
   const char *repo = NULL;
+  const char *sysroot_repo_path = "/sysroot/ostree/repo";
+  const char *host_repo_path = "/ostree/repo";
   GFile *repo_file = NULL;
   int arg_off;
 
@@ -105,20 +102,24 @@ ostree_main (int    argc,
   g_set_prgname (argv[0]);
 
   if (argc < 2)
-    return usage (argv, builtins, 1);
+    return ostree_usage (argv, commands, TRUE);
 
-  am_root = getuid () == 0;
   have_repo_arg = g_str_has_prefix (argv[1], "--repo=");
 
-  if (!have_repo_arg && am_root)
-    repo = "/sysroot/ostree/repo";
-  else if (have_repo_arg)
-    repo = argv[1] + strlen ("--repo=");
+  if (have_repo_arg)
+    {
+      repo = argv[1] + strlen ("--repo=");
+    }
   else
-    repo = NULL;
+    {
+      if (g_file_test (sysroot_repo_path, G_FILE_TEST_EXISTS))
+        repo = sysroot_repo_path;
+      else if (g_file_test (host_repo_path, G_FILE_TEST_EXISTS))
+        repo = host_repo_path;
+    }
 
   if (repo)
-    repo_file = ot_gfile_new_for_path (repo);
+    repo_file = g_file_new_for_path (repo);
 
   slash = strrchr (argv[0], '/');
   if (slash)
@@ -126,6 +127,9 @@ ostree_main (int    argc,
   else
     binname = argv[0];
 
+  if (g_str_has_prefix (binname, "lt-"))
+    binname += 3;
+
   if (g_str_has_prefix (binname, "ostree-"))
     {
       cmd = strchr (binname, '-');
@@ -146,30 +150,32 @@ ostree_main (int    argc,
       cmd = argv[arg_off-1];
     }
 
-  builtin = builtins;
-  while (builtin->name)
+  command = commands;
+  while (command->name)
     {
-      if (g_strcmp0 (cmd, builtin->name) == 0)
+      if (g_strcmp0 (cmd, command->name) == 0)
         break;
-      builtin++;
+      command++;
     }
 
-  if (!builtin->name)
+  if (!command->fn)
     {
       ot_lfree char *msg = g_strdup_printf ("Unknown command '%s'", cmd);
-      set_error_print_usage (&error, builtins, msg, argv);
+      g_set_error_literal (&error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, msg);
       goto out;
     }
 
-  if (repo == NULL && !(builtin->flags & OSTREE_BUILTIN_FLAG_NO_REPO))
+  if (repo == NULL && !(command->flags & OSTREE_BUILTIN_FLAG_NO_REPO))
     {
-      set_error_print_usage (&error, builtins, "Command requires a --repo argument", argv);
+      g_set_error_literal (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                           "Command requires a --repo argument");
+      ostree_usage (argv, commands, TRUE);
       goto out;
     }
   
-  prep_builtin_argv (cmd, argc-arg_off, argv+arg_off, &cmd_argc, &cmd_argv);
+  ostree_prep_builtin_argv (cmd, argc-arg_off, argv+arg_off, &cmd_argc, &cmd_argv);
 
-  if (!builtin->fn (cmd_argc, cmd_argv, repo_file, &error))
+  if (!command->fn (cmd_argc, cmd_argv, repo_file, &error))
     goto out;
 
  out:
@@ -177,9 +183,30 @@ ostree_main (int    argc,
   g_clear_object (&repo_file);
   if (error)
     {
-      g_printerr ("%s\n", error->message);
-      g_clear_error (&error);
+      g_propagate_error (res_error, error);
       return 1;
     }
   return 0;
 }
+
+int
+ostree_main (int    argc,
+             char **argv,
+             OstreeCommand *commands)
+{
+  GError *error = NULL;
+  int ret;
+
+  ret = ostree_run (argc, argv, commands, &error);
+
+  if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED))
+    ostree_usage (argv, commands, TRUE);
+
+  if (error)
+    {
+      g_printerr ("%s\n", error->message);
+      g_error_free (error);
+    }
+
+  return ret;
+}
